package be.iminds.ilabt.jfed.experimenter_gui.config;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.UserInfo;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.UserInfoBuilder;
import be.iminds.ilabt.jfed.preferences.CorePreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.common.GeniUrn;
import be.iminds.ilabt.jfed.util.library.JFedVersionInfo;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import javax.inject.Singleton;
import java.util.*;
import java.util.stream.Collectors;

/**
 * User: twalcari
 * Date: 4/25/2016
 * Time: 14:52
 */
@Singleton
public class UserInfoProvider {
    @Nonnull private final JFedVersionInfo jFedVersionInfo;
    @Nonnull private List<String> extraFlags;

    @Inject
    public UserInfoProvider(@Nonnull JFedVersionInfo jFedVersionInfo, @Nonnull JFedPreferences jFedPreferences) {
        this.jFedVersionInfo = jFedVersionInfo;

        extraFlags = Collections.unmodifiableList(jFedPreferences.getStringList(CorePreferenceKey.PREF_GUICONFIG_EXTRA_FLAGS));
    }

    public void addExtraFlags(@Nonnull String... extraExtraFlags) {
        addExtraFlags(Arrays.asList(extraExtraFlags));
    }
    public void addExtraFlags(@Nonnull List<String> extraExtraFlags) {
        Set<String> res = new TreeSet<>(extraFlags);
        res.addAll(extraExtraFlags);
        extraFlags = Collections.unmodifiableList(new ArrayList<>(res));
    }

    public UserInfo createUserInfo(@Nullable GeniUrn userUrn, @Nullable String... extraExtraFlags){
        Set<String> usedExtraFlags = new TreeSet<>(extraFlags);
        if (extraExtraFlags != null) {
            Arrays.stream(extraExtraFlags).filter(Objects::nonNull).forEach(usedExtraFlags::add);
        }

        UserInfoBuilder userInfoBuilder = new UserInfoBuilder();
        userInfoBuilder.setUrn(userUrn);

        userInfoBuilder.setVersionJava(System.getProperty("java.version"));
        userInfoBuilder.setVersionJavaVendor(System.getProperty("java.vendor"));
        userInfoBuilder.setVersionJavaSpec(System.getProperty("java.specification.version"));

        userInfoBuilder.setVersionOs(System.getProperty("os.name") + " " + System.getProperty("os.version"));
        userInfoBuilder.setVersionArch(System.getProperty("os.arch"));

        userInfoBuilder.setVersionJFed(jFedVersionInfo.getFullVersionString());

        userInfoBuilder.setExtra(new ArrayList<>(usedExtraFlags));
        return userInfoBuilder.create();
    }


}
